@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # vim:et sts=4 sw=4
 #
 # ibus-table - The Tables engine for IBus
@@ -31,6 +30,7 @@ from typing import Callable
 from enum import Enum, Flag
 import sys
 import os
+import re
 import logging
 import gettext
 from gi import require_version # type: ignore
@@ -119,7 +119,7 @@ def color_string_to_argb(color_string: str) -> int:
     return (((int(gdk_rgba.alpha * 0xff) & 0xff) << 24)
             + ((int(gdk_rgba.red * 0xff) & 0xff) << 16)
             + ((int(gdk_rgba.green * 0xff) & 0xff) << 8)
-            + ((int(gdk_rgba.blue * 0xff) & 0xff)))
+            + (int(gdk_rgba.blue * 0xff) & 0xff))
 
 def get_default_chinese_mode(database: tabsqlitedb.TabSqliteDb) -> int:
     '''
@@ -403,7 +403,7 @@ class Capabilite(Flag):
             return int(self) | other
         return NotImplemented
 
-    def __ror__(self, other: Any) -> Any:
+    def __ror__(self, other: Any) -> Any: # type: ignore[override]
         return self.__or__(other)
 
     def __and__(self, other: Any) -> Any:
@@ -415,7 +415,7 @@ class Capabilite(Flag):
             return int(self) & other
         return NotImplemented
 
-    def __rand__(self, other: Any) -> Any:
+    def __rand__(self, other: Any) -> Any: # type: ignore[override]
         return self.__and__(other)
 
     PREEDIT_TEXT = ('PREEDIT_TEXT')
@@ -617,7 +617,7 @@ class InputHints(Flag):
             return int(self) | other
         return NotImplemented
 
-    def __ror__(self, other: Any) -> Any:
+    def __ror__(self, other: Any) -> Any: # type: ignore[override]
         return self.__or__(other)
 
     def __and__(self, other: Any) -> Any:
@@ -630,7 +630,7 @@ class InputHints(Flag):
             return int(self) & other
         return NotImplemented
 
-    def __rand__(self, other: Any) -> Any:
+    def __rand__(self, other: Any) -> Any: # type: ignore[override]
         return self.__and__(other)
 
     NONE = ('NONE')
@@ -656,6 +656,21 @@ class KeyEvent:
         self.code = keycode
         self.state = state
         self.name = IBus.keyval_name(self.val)
+        if re.match(r'U\+[0-9a-fA-F]{4,5}', self.name):
+            # Older versions of ibus produce
+            #
+            # >>> IBus.keyval_name(0x0100263a)
+            # 'U+263A'
+            #
+            # Convert this into the same name used by newere
+            # versions of ibus which is:
+            #
+            # >>> IBus.keyval_name(0x0100263a)
+            # '0x100263a'
+            #
+            # to make it possible for me to always work
+            # with the same names, no matter the ibus version
+            self.name = f'0x{0x1000000  + int(self.name[2:], 16):x}'
         self.unicode = IBus.keyval_to_unicode(self.val)
         self.shift = self.state & IBus.ModifierType.SHIFT_MASK != 0
         self.lock = self.state & IBus.ModifierType.LOCK_MASK != 0
@@ -702,7 +717,7 @@ class KeyEvent:
 
     def __str__(self) -> str:
         return repr(
-            f'val={self.val} '
+            f'val={self.val:08x} '
             f'code={self.code} '
             f'state=0x{self.state:08x} '
             f'name=“{self.name}” '
@@ -727,6 +742,30 @@ class KeyEvent:
             f'modifier={self.modifier}')
 
 def keyevent_to_keybinding(keyevent: KeyEvent) -> str:
+    # pylint: disable=line-too-long
+    '''Calculates a keybinding string from a key event.
+
+    Examples:
+
+    >>> keyevent_to_keybinding(KeyEvent(IBus.KEY_Left, 0, 0))
+    'Left'
+
+    >>> keyevent_to_keybinding(KeyEvent(0x0100263a, 0, 0))
+    '0x100263a'
+
+    >>> keyevent_to_keybinding(KeyEvent(0x0101F923, 0, 0))
+    '0x101f923'
+
+    >>> keyevent_to_keybinding(KeyEvent(IBus.KEY_Left, 0, IBus.ModifierType.SHIFT_MASK | IBus.ModifierType.CONTROL_MASK))
+    'Shift+Control+Left'
+
+    >>> keyevent_to_keybinding(KeyEvent(0x0100263A, 0, IBus.ModifierType.SHIFT_MASK | IBus.ModifierType.CONTROL_MASK))
+    'Shift+Control+0x100263a'
+
+    >>> keyevent_to_keybinding(KeyEvent(0x0101F923, 0, IBus.ModifierType.SHIFT_MASK | IBus.ModifierType.CONTROL_MASK))
+    'Shift+Control+0x101f923'
+    '''
+    # pylint: enable=line-too-long
     keybinding = ''
     if keyevent.shift:
         keybinding += 'Shift+'
@@ -754,8 +793,37 @@ def keyevent_to_keybinding(keyevent: KeyEvent) -> str:
     return keybinding
 
 def keybinding_to_keyevent(keybinding: str) -> KeyEvent:
+    # pylint: disable=line-too-long
+    '''Returns a key event object created from a key binding string.
+
+    Examples:
+
+    >>> keybinding_to_keyevent('Shift+Control+Left').val == IBus.KEY_Left
+    True
+
+    >>> keybinding_to_keyevent('Shift+Control+Left').name
+    'Left'
+
+    >>> keybinding_to_keyevent('Shift+Control+Left').state == IBus.ModifierType.SHIFT_MASK | IBus.ModifierType.CONTROL_MASK
+    True
+
+    >>> f"0x{keybinding_to_keyevent('Shift+Control+0x100263a').val:08x}"
+    '0x0100263a'
+
+    >>> f"0x{keybinding_to_keyevent('Shift+Control+0x101F923').val:08x}"
+    '0x0101f923'
+
+    >>> f"0x{keybinding_to_keyevent('Shift+Control+0x101G923').val:08x}"
+    '0x00ffffff'
+
+    >>> keybinding_to_keyevent('Shift+Control+0x101G923').val == IBus.KEY_VoidSymbol
+    True
+    '''
+    # pylint: enable=line-too-long
     name = keybinding.split('+')[-1]
     keyval = IBus.keyval_from_name(name)
+    if keyval == IBus.KEY_VoidSymbol and re.match(r'0x10[0-9a-fA-F]{5}', name):
+        keyval = int(name[2:], 16)
     state = 0
     if 'Shift+' in keybinding:
         state |= IBus.ModifierType.SHIFT_MASK
